@@ -1,11 +1,25 @@
 /**
  * @file
  * @brief abstract graph C library, @ref cgraph_api
+ * @ingroup cgraph_api
+ *
+ * **Libcgraph** supports graph programming by maintaining graphs
+ * in memory and reading and writing graph files.
+ * Graphs are composed of nodes, edges, and nested subgraphs.
+ * These graph objects may be attributed with string name-value pairs
+ * and programmer-defined records (see Attributes).
+ * All of Libcgraph’s global symbols have the prefix **ag** (case varying).
+ * In the following, if a function has a parameter `int createflag` and
+ * the object does not exist, the function will create the specified object
+ * if `createflag` is non-zero; otherwise, it will return NULL.
  *
  * [man 3 cgraph](https://graphviz.org/pdf/cgraph.3.pdf)
  *
  * @defgroup cgraph_api Cgraph API
- * @brief Abstract graph C library
+ * @brief Abstract graph C library. API cgraph.h
+ * @ingroup public_apis
+ *
+ * [man 3 cgraph](https://graphviz.org/pdf/cgraph.3.pdf)
  *
  * Main types @ref Agraph_t, @ref Agnode_t, @ref Agedge_t.
  * @{
@@ -24,7 +38,10 @@
 #pragma once
 
 #include <inttypes.h>
+#include <stdbool.h>
+#include <stdio.h>
 #include "cdt.h"
+#include <stddef.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -44,25 +61,26 @@ extern "C" {
 #define CGRAPH_API /* nothing */
 #endif
 
-#ifndef FALSE
-#define FALSE (0)
-#endif
-#ifndef TRUE
-#define TRUE (!FALSE)
-#endif
-
 /// @endcond
 
+/// @addtogroup cgraph_misc
+/// @{
 typedef uint64_t IDTYPE;
 
 /* forward struct type declarations */
 typedef struct Agtag_s Agtag_t;
 typedef struct Agobj_s Agobj_t;         ///< generic object header
+/// @}
+/// @ingroup cgraph_graph
 typedef struct Agraph_s Agraph_t;       ///< graph, subgraph (or hyperedge)
+/// @ingroup cgraph_node
 typedef struct Agnode_s Agnode_t;       ///< node (atom)
+/// @ingroup cgraph_edge
 typedef struct Agedge_s Agedge_t;       ///< node pair
+/// @ingroup cgraph_graph
 typedef struct Agdesc_s Agdesc_t;       ///< graph descriptor
-typedef struct Agmemdisc_s Agmemdisc_t; ///< memory allocator
+/// @addtogroup cgraph_misc
+/// @{
 typedef struct Agiddisc_s Agiddisc_t;   ///< object ID allocator
 typedef struct Agiodisc_s Agiodisc_t;   ///< IO services
 typedef struct Agdisc_s Agdisc_t;       ///< union of client discipline methods
@@ -70,37 +88,102 @@ typedef struct Agdstate_s Agdstate_t;   ///< client state (closures)
 typedef struct Agsym_s Agsym_t;         ///< string attribute descriptors
 typedef struct Agattr_s Agattr_t;       ///< string attribute container
 typedef struct Agcbdisc_s Agcbdisc_t;   ///< client event callbacks
-typedef struct Agcbstack_s Agcbstack_t; ///< enclosing state for cbdisc
+typedef struct Agcbstack_s Agcbstack_t; ///< enclosing state for Agcbdisc_t
 typedef struct Agclos_s Agclos_t;       ///< common fields for graph/subgs
-typedef struct Agrec_s Agrec_t;         ///< generic runtime record
 typedef struct Agdatadict_s Agdatadict_t; ///< set of dictionaries per graph
 typedef struct Agedgepair_s Agedgepair_t; ///< the edge object
 typedef struct Agsubnode_s Agsubnode_t;
+/// @}
 
-/** @brief Header of a user record.
+/** @addtogroup cgraph_attr
+ *  @{
+ *
+ *  @defgroup cgraph_rec records
+ *  @brief These records are attached by client programs dynamically at runtime.
+ *  @{
+ *
+ *  Uninterpreted records may be attached to graphs, subgraphs, nodes,
+ *  and edges for efficient operations on values such as marks, weights,
+ *  counts, and pointers needed by algorithms.
+ *  Application programmers define the fields of these records,
+ *  but they must be declared with a common record header @ref Agrec_t.
+ *
+ *  A unique string ID (stored in @ref Agrec_s.name) must be given
+ *  to each record attached to the same object.
+ *  Cgraph has functions to create, search for, and delete these records.
+ *  The records are maintained in a circular list,
+ *  with @ref Agobj_s.data pointing somewhere in the list.
+ *  The search function @ref aggetrec has an option to lock this pointer on a given record.
+ *  The application must be written so only one such lock is outstanding at a time.
+ *
+ *  Records are created and managed by Libcgraph.
+ *  A programmer must explicitly attach them to the objects in a graph,
+ *  either to individual objects one at a time via @ref agbindrec,
+ *  or to all the objects of the same class in a graph via @ref aginit.
+ *  The `name` argument of a record distinguishes various types of records,
+ *  and is programmer defined.
+ *
+ *  Libcgraph reserves the prefix "_AG_" (in
+ *  @ref DataDictName,
+ *  @ref AgDataRecName,
+ *  @ref DRName).
+ *
+ *  Internally, records Agrec_s are maintained in circular linked lists
+ *  attached to graph objects Agobj_s.
+ *  To allow referencing application-dependent data without function calls or search,
+ *  Libcgraph allows setting and locking the list pointer
+ *  of a graph, node, or edge on a particular record
+ *  (see @ref Agtag_s.mtflock and @ref Agobj_s.data).
+ *  This pointer can be obtained with the macro @ref AGDATA(obj).
+ *  A cast, generally within a macro or inline function,
+ *  is usually applied to convert the list pointer to
+ *  an appropriate programmer-defined type (eg. @ref GD_parent).
+ *
+ *  To control the setting of this pointer,
+ *  the `move_to_front` flag may be TRUE or FALSE.
+ *  If `move_to_front` is TRUE, the record will be
+ *  locked @ref Agtag_s.mtflock at the
+ *  head of the list @ref Agobj_s.data,
+ *  so it can be accessed directly by @ref AGDATA(obj).
+ *
+ *  The lock protects the data pointer from being moved.
+ *  Function @ref aggetrec reports error when data pointer and lock
+ *  are reassigned.
+ *
+ *  The lock can be released or reset by a call to @ref agdelrec.
+ *
+ */
 
-These records are attached by client programs
-dynamically at runtime.  A unique string ID must be given to each record
-attached to the same object.  Cgraph has functions to create, search for,
-and delete these records.   The records are maintained in a circular list,
-with obj->data pointing somewhere in the list.  The search function has
-an option to lock this pointer on a given record.  The application must
-be written so only one such lock is outstanding at a time. */
+typedef struct Agrec_s Agrec_t;
+///< generic header of @ref Agattr_s, @ref Agdatadict_s and user records
 
+/// implementation of @ref Agrec_t
 struct Agrec_s {
     char *name;
-    Agrec_t *next;
+    Agrec_t *next; ///< **circular** linked list of records
     /* following this would be any programmer-defined data */
 };
+/// @}
+/// @}
 
-/** @brief Object tag for graphs, nodes, and edges.
+/** @defgroup cgraph_object objects
+ *  @brief parent for @ref cgraph_graph, @ref cgraph_node, and @ref cgraph_edge.
+ *  @ingroup cgraph_api
+ *
+ * Common parameter for functions:
+ * @param obj generic pointer to @ref Agraph_t, @ref Agnode_t or @ref Agedge_t
+ * @{
+ */
+
+/** @brief tag in @ref Agobj_s for graphs, nodes, and edges.
 
 While there may be several structs
 for a given node or edges, there is only one unique ID (per main graph).  */
 
 struct Agtag_s {
+    /// access with @ref AGTYPE
     unsigned objtype:2;		/* see literal tags below */
-    unsigned mtflock:1;		/* move-to-front lock, see above */
+    unsigned mtflock:1; ///< @brief move-to-front lock, guards @ref Agobj_s.data
     unsigned attrwf:1;		/* attrs written (parity, write.c) */
     unsigned seq:(sizeof(unsigned) * 8 - 4);	/* sequence no. */
     IDTYPE id;		        /* client  ID */
@@ -113,10 +196,10 @@ struct Agtag_s {
 #define AGINEDGE			3	/* (1 << 1) indicates an edge tag.   */
 #define AGEDGE 				AGOUTEDGE	/* synonym in object kind args */
 
-/// a generic graph/node/edge header
+/// a generic header of @ref Agraph_s, @ref Agnode_s and @ref Agedge_s
 struct Agobj_s {
-    Agtag_t tag;
-    Agrec_t *data;
+    Agtag_t tag;   ///< access with @ref AGTAG
+    Agrec_t *data; ///< stores programmer-defined data, access with @ref AGDATA
 };
 
 #define AGTAG(obj)		(((Agobj_t*)(obj))->tag)
@@ -125,6 +208,18 @@ struct Agobj_s {
 #define AGSEQ(obj)		(AGTAG(obj).seq)
 #define AGATTRWF(obj)		(AGTAG(obj).attrwf)
 #define AGDATA(obj)		(((Agobj_t*)(obj))->data)
+/// @}
+/// @} cgraph_api
+
+/** @defgroup cgraph_node nodes
+ *  @ingroup cgraph_object
+ *
+ * A node is created by giving a unique string name or programmer
+ * defined integer ID, and is represented by a unique internal object.
+ * (%Node equality can checked by pointer comparison.)
+ *
+ * @{
+ */
 
 /** @brief This is the node struct allocated per graph (or subgraph).
 
@@ -146,7 +241,10 @@ struct Agnode_s {
     Agraph_t *root;
     Agsubnode_t mainsub;	/* embedded for main graph */
 };
+/// @}
 
+/// @addtogroup cgraph_edge
+/// @{
 struct Agedge_s {
     Agobj_t base;
     Dtlink_t id_link;		/* main graph only */
@@ -157,32 +255,44 @@ struct Agedge_s {
 struct Agedgepair_s {
     Agedge_t out, in;
 };
+/// @}
 
+/// @addtogroup cgraph_graph
+/// @{
+
+/// graph descriptor
 struct Agdesc_s {		/* graph descriptor */
     unsigned directed:1;	/* if edges are asymmetric */
     unsigned strict:1;		/* if multi-edges forbidden */
     unsigned no_loop:1;		/* if no loops */
     unsigned maingraph:1;	/* if this is the top level graph */
-    unsigned flatlock:1;	/* if sets are flattened into lists in cdt */
     unsigned no_write:1;	/* if a temporary subgraph */
     unsigned has_attrs:1;	/* if string attr tables should be initialized */
     unsigned has_cmpnd:1;	/* if may contain collapsed nodes */
 };
+/// @}
 
-/* disciplines for external resources needed by libgraph */
+/** @defgroup cgraph_disc disciplines
+ *  @ingroup cgraph_misc
+ *  @brief disciplines for external resources needed by libgraph
+ *
+ *  (This section is not intended for casual users.)
+ *
+ *  Programmer-defined disciplines customize certain resources:
+ *  ID namespace, memory, and I/O - needed by Libcgraph.
+ *  A discipline struct (or NULL) is passed at graph creation time.
+ *  @{
+ */
 
-///  memory allocator discipline, independent of other resources
-
-struct Agmemdisc_s {
-    void *(*open) (Agdisc_t*);
-    void *(*alloc) (void *state, size_t req);
-    void *(*resize) (void *state, void *ptr, size_t old, size_t req);
-    void (*free) (void *state, void *ptr);
-    void (*close) (void *state);
-};
-
-/// object ID allocator discipline
+/**
+ * @brief object ID allocator discipline
+ *
+ * An ID allocator discipline allows a client to control assignment
+ * of IDs (uninterpreted integer values) to objects, and possibly how
+ * they are mapped to and from strings.
+ */
 
+/// object ID allocator
 struct Agiddisc_s {
     void *(*open) (Agraph_t * g, Agdisc_t*);	/* associated with a graph */
     long (*map) (void *state, int objtype, char *str, IDTYPE *id,
@@ -194,6 +304,7 @@ struct Agiddisc_s {
     void (*idregister) (void *state, int objtype, void *obj);
 };
 
+/// IO services
 struct Agiodisc_s {
     int (*afread) (void *chan, char *buf, int bufsize);
     int (*putstr) (void *chan, const char *str);
@@ -201,24 +312,29 @@ struct Agiodisc_s {
     /* error messages? */
 };
 
-/// user's discipline
-
+/// @brief user's discipline
+///
+/// A default discipline is supplied when NULL is given for any of these fields.
 struct Agdisc_s {
-    Agmemdisc_t *mem;
     Agiddisc_t *id;
     Agiodisc_t *io;
 };
 
 	/* default resource disciplines */
 
-CGRAPH_API extern Agmemdisc_t AgMemDisc;
 CGRAPH_API extern Agiddisc_t AgIdDisc;
 CGRAPH_API extern Agiodisc_t AgIoDisc;
 
 CGRAPH_API extern Agdisc_t AgDefaultDisc;
+/// @}
 
+/** @defgroup cgraph_graph graphs
+ *  @ingroup cgraph_object
+ *  @{
+ */
+
+/// client state (closures)
 struct Agdstate_s {
-    void *mem;
     void *id;
     /* IO must be initialized and finalized outside Cgraph,
      * and channels (FILES) are passed as void* arguments. */
@@ -228,6 +344,7 @@ typedef void (*agobjfn_t) (Agraph_t * g, Agobj_t * obj, void *arg);
 typedef void (*agobjupdfn_t) (Agraph_t * g, Agobj_t * obj, void *arg,
 			      Agsym_t * sym);
 
+/// client event callbacks, used in Agcbstack_s
 struct Agcbdisc_s {
     struct {
 	agobjfn_t ins;
@@ -238,55 +355,88 @@ struct Agcbdisc_s {
 
 /// object event callbacks
 
+/// enclosing state for Agcbdisc_s, used in Agclos_s
 struct Agcbstack_s {
     Agcbdisc_t *f;		/* methods */
     void *state;		/* closure */
     Agcbstack_t *prev;		/* kept in a stack, unlike other disciplines */
 };
 
+/// shared resources for Agraph_s
 struct Agclos_s {
     Agdisc_t disc;		/* resource discipline functions */
     Agdstate_t state;		/* resource closures */
     Dict_t *strdict;		/* shared string dict */
     uint64_t seq[3];	/* local object sequence number counter */
     Agcbstack_t *cb;		/* user and system callback function stacks */
-    unsigned char callbacks_enabled;	/* issue user callbacks or hold them? */
     Dict_t *lookup_by_name[3];
     Dict_t *lookup_by_id[3];
 };
 
+/// graph or subgraph
 struct Agraph_s {
     Agobj_t base;
     Agdesc_t desc;
-    Dtlink_t link;
+    Dtlink_t seq_link;
+    Dtlink_t id_link;
     Dict_t *n_seq;		/* the node set in sequence */
     Dict_t *n_id;		/* the node set indexed by ID */
     Dict_t *e_seq, *e_id;	/* holders for edge sets */
-    Dict_t *g_dict;		/* subgraphs - descendants */
+    Dict_t *g_seq, *g_id;	/* subgraphs - descendants */
     Agraph_t *parent, *root;	/* subgraphs - ancestors */
     Agclos_t *clos;		/* shared resources */
 };
 
 CGRAPH_API void agpushdisc(Agraph_t * g, Agcbdisc_t * disc, void *state);
 CGRAPH_API int agpopdisc(Agraph_t * g, Agcbdisc_t * disc);
-CGRAPH_API int agcallbacks(Agraph_t * g, int flag);	/* return prev value */
 
 /* graphs */
 CGRAPH_API Agraph_t *agopen(char *name, Agdesc_t desc, Agdisc_t * disc);
+/**<
+ * @brief creates a new graph with the given name and kind
+ *
+ * @param desc - graph kind, can be @ref Agdirected, @ref Agundirected,
+ * @ref Agstrictdirected or @ref Agstrictundirected.
+ * A strict graph cannot have multi-edges or self-arcs.
+ *
+ * @param disc - discipline structure which can be used
+ * to tailor I/O, memory allocation, and ID allocation. Typically, a NULL
+ * value will be used to indicate the default discipline @ref AgDefaultDisc.
+ */
+
 CGRAPH_API int agclose(Agraph_t * g);
+///< deletes a graph, freeing its associated storage
 CGRAPH_API Agraph_t *agread(void *chan, Agdisc_t * disc);
+///< constructs a new graph
+
 CGRAPH_API Agraph_t *agmemread(const char *cp);
+///< reads a graph from the input string
+
 CGRAPH_API Agraph_t *agmemconcat(Agraph_t *g, const char *cp);
 CGRAPH_API void agreadline(int);
+///< sets input line number for subsequent error reporting
+
 CGRAPH_API void agsetfile(const char *);
+///< sets the current file name for subsequent error reporting
+
 CGRAPH_API Agraph_t *agconcat(Agraph_t * g, void *chan, Agdisc_t * disc);
+/**< @brief merges the file contents with a pre-existing graph
+ *
+ * Though I/O methods may be overridden, the default is that
+ * the channel argument is a stdio FILE pointer.
+ * In that case, if any of the streams are wide-oriented,
+ * the behavior is undefined.
+ */
+
 CGRAPH_API int agwrite(Agraph_t * g, void *chan);
 CGRAPH_API int agisdirected(Agraph_t * g);
 CGRAPH_API int agisundirected(Agraph_t * g);
 CGRAPH_API int agisstrict(Agraph_t * g);
 CGRAPH_API int agissimple(Agraph_t * g);
+/// @}
 
-/* nodes */
+/// @addtogroup cgraph_node
+/// @{
 CGRAPH_API Agnode_t *agnode(Agraph_t * g, char *name, int createflag);
 CGRAPH_API Agnode_t *agidnode(Agraph_t * g, IDTYPE id, int createflag);
 CGRAPH_API Agnode_t *agsubnode(Agraph_t * g, Agnode_t * n, int createflag);
@@ -297,8 +447,29 @@ CGRAPH_API Agnode_t *agprvnode(Agraph_t * g, Agnode_t * n);
 
 CGRAPH_API Agsubnode_t *agsubrep(Agraph_t * g, Agnode_t * n);
 CGRAPH_API int agnodebefore(Agnode_t *u, Agnode_t *v); /* we have no shame */
+/// @}
+
+/** @defgroup cgraph_edge edges
+ *  @ingroup cgraph_object
+ *
+ * An abstract edge has two endpoint nodes called tail and head
+ * where all outedges of the same node have it as the tail
+ * value and similarly all inedges have it as the head.
+ * In an undirected graph, head and tail are interchangeable.
+ * If a graph has multi-edges between the same pair of nodes,
+ * the edge's string name behaves as a secondary key.
+ *
+ * Note that an abstract edge has two distinct concrete
+ * representations: as an in-edge and as an out-edge.
+ * In particular, the pointer as an out-edge is different
+ * from the pointer as an in-edge.
+ * The function @ref ageqedge canonicalizes the pointers before
+ * doing a comparison and so can be used to test edge equality.
+ * The sense of an edge can be flipped using @ref agopp.
+ *
+ * @{
+ */
 
-/* edges */
 CGRAPH_API Agedge_t *agedge(Agraph_t * g, Agnode_t * t, Agnode_t * h,
 			char *name, int createflag);
 CGRAPH_API Agedge_t *agidedge(Agraph_t * g, Agnode_t * t, Agnode_t * h,
@@ -310,8 +481,13 @@ CGRAPH_API Agedge_t *agfstout(Agraph_t * g, Agnode_t * n);
 CGRAPH_API Agedge_t *agnxtout(Agraph_t * g, Agedge_t * e);
 CGRAPH_API Agedge_t *agfstedge(Agraph_t * g, Agnode_t * n);
 CGRAPH_API Agedge_t *agnxtedge(Agraph_t * g, Agedge_t * e, Agnode_t * n);
+/// @}
+
+/// @addtogroup cgraph_object
+/// @{
+
+// Generic object (graphs, nodes and edges) functions
 
-/* generic */
 CGRAPH_API Agraph_t *agraphof(void* obj);
 CGRAPH_API Agraph_t *agroot(void* obj);
 CGRAPH_API int agcontains(Agraph_t *, void *);
@@ -322,8 +498,17 @@ CGRAPH_API int agdelsubg(Agraph_t * g, Agraph_t * sub);	/* could be agclose */
 CGRAPH_API int agdelnode(Agraph_t * g, Agnode_t * arg_n);
 CGRAPH_API int agdeledge(Agraph_t * g, Agedge_t * arg_e);
 CGRAPH_API int agobjkind(void *);
+/// @}
 
-/* strings */
+/** @defgroup cgraph_string string utilities
+ *  @brief reference-counted strings
+ *  @ingroup cgraph_misc
+ *
+ *  Storage management of strings as reference-counted strings.
+ *  The caller does not need to dynamically allocate storage.
+ *
+ * @{
+ */
 CGRAPH_API char *agstrdup(Agraph_t *, const char *);
 CGRAPH_API char *agstrdup_html(Agraph_t *, const char *);
 CGRAPH_API int aghtmlstr(const char *);
@@ -332,28 +517,51 @@ CGRAPH_API int agstrfree(Agraph_t *, const char *);
 CGRAPH_API char *agcanon(char *, int);
 CGRAPH_API char *agstrcanon(char *, char *);
 CGRAPH_API char *agcanonStr(char *str);  /* manages its own buf */
+/// @}
 
-/// definitions for dynamic string attributes
+/** @defgroup cgraph_attr attributes
+ *  @brief symbols, and @ref cgraph_rec
+ *  @ingroup cgraph_api
+ *
+ * Programmer-defined values may be dynamically
+ * attached to graphs, subgraphs, nodes, and edges.
+ * Such values are either character string data (see @ref agattr) (for I/O)
+ * or uninterpreted binary @ref cgraph_rec (for implementing algorithms efficiently).
+ *
+ * *String attributes* are handled automatically in reading and writing graph files.
+ * A string attribute is identified by name
+ * and by an internal symbol table entry (@ref Agsym_t) created by Libcgraph.
+ * Attributes of nodes, edges, and graphs (with their subgraphs) have separate namespaces.
+ * The contents of an @ref Agsym_t have a char* *name* for the attribute's name,
+ * a char* *defval* field for the attribute's default value,
+ * and an int *id* field containing the index of the attribute's specific value
+ * for an object in the object's array of attribute values.
+ *
+ * @{
+ */
 
+// definitions for dynamic string attributes
+
+/// string attribute container
 struct Agattr_s {		/* dynamic string attributes */
     Agrec_t h;			/* common data header */
     Dict_t *dict;		/* shared dict to interpret attr field */
-    char **str;			/* the attribute string values */
+    char **str;		///< the attribute string values indexed by Agsym_s.id
 };
 
-/// symbol in one of the above dictionaries
-
+/// @brief string attribute descriptor
+/// symbol in Agattr_s.dict
 struct Agsym_s {
     Dtlink_t link;
     char *name;			/* attribute's name */
     char *defval;		/* its default value for initialization */
-    int id;			/* its index in attr[] */
+    int id;			///< index in Agattr_s.str
     unsigned char kind;		/* referent object type */
     unsigned char fixed;	/* immutable value */
     unsigned char print;	/* always print */
 };
 
-struct Agdatadict_s {		/* set of dictionaries per graph */
+struct Agdatadict_s {		///< set of dictionaries per graph
     Agrec_t h;			/* installed in list of graph recs */
     struct {
 	Dict_t *n, *e, *g;
@@ -362,17 +570,52 @@ struct Agdatadict_s {		/* set of dictionaries per graph */
 
 CGRAPH_API Agsym_t *agattr(Agraph_t * g, int kind, char *name,
                            const char *value);
+/**< @brief creates or looks up attributes of a graph
+ * @param g graph. When is NULL, the default is set for all graphs created subsequently.
+ * @param kind may be @ref AGRAPH, @ref AGNODE, or @ref AGEDGE.
+ * @param value default value. When is (char*)0, the request is to search
+ * for an existing attribute of the given kind and name.
+ *
+ * If the attribute already exists, its default
+ * for creating new objects is set to the given **value**;
+ * if it does not exist, a new attribute is created with the
+ * given default **value**, and the default is applied to all pre-existing
+ * objects of the given **kind**
+ */
+
 CGRAPH_API Agsym_t *agattrsym(void *obj, char *name);
+///< looks up a string attribute for a graph object given as an argument
+
 CGRAPH_API Agsym_t *agnxtattr(Agraph_t * g, int kind, Agsym_t * attr);
 CGRAPH_API int      agcopyattr(void *oldobj, void *newobj);
 
+/// @addtogroup cgraph_rec
+/// @{
 CGRAPH_API void *agbindrec(void *obj, const char *name, unsigned int recsize,
 		       int move_to_front);
+///< @brief attach a new record of the given size to the object
+/// @param recsize if 0, the call to @ref agbindrec is simply a lookup
+
 CGRAPH_API Agrec_t *aggetrec(void *obj, const char *name, int move_to_front);
+///< find record in circular list and do optional move-to-front and lock
+
 CGRAPH_API int agdelrec(void *obj, const char *name);
+///<  deletes a named record from one object
+
 CGRAPH_API void aginit(Agraph_t * g, int kind, const char *rec_name,
                        int rec_size, int move_to_front);
+/**< @brief attach new records to objects of specified kind
+ *
+ * @param kind may be @ref AGRAPH, @ref AGNODE, or @ref AGEDGE
+ * @param rec_size if is negative (of the actual rec_size) for graphs,
+ * @ref aginit is applied recursively to the graph and its subgraphs
+ */
+
 CGRAPH_API void agclean(Agraph_t * g, int kind, char *rec_name);
+///< @brief calls @ref agdelrec for all objects
+/// of the same class in an entire graph
+
+/// @}
 
 CGRAPH_API char *agget(void *obj, char *name);
 CGRAPH_API char *agxget(void *obj, Agsym_t * sym);
@@ -380,22 +623,34 @@ CGRAPH_API int agset(void *obj, char *name, const char *value);
 CGRAPH_API int agxset(void *obj, Agsym_t * sym, const char *value);
 CGRAPH_API int agsafeset(void* obj, char* name, const char* value,
                          const char* def);
+/// @}
 
-/* definitions for subgraphs */
+/// @defgroup cgraph_subgraph subgraphs
+/// @ingroup cgraph_graph
+/// @{
 CGRAPH_API Agraph_t *agsubg(Agraph_t * g, char *name, int cflag);	/* constructor */
 CGRAPH_API Agraph_t *agidsubg(Agraph_t * g, IDTYPE id, int cflag);	/* constructor */
 CGRAPH_API Agraph_t *agfstsubg(Agraph_t * g);
 CGRAPH_API Agraph_t *agnxtsubg(Agraph_t * subg);
 CGRAPH_API Agraph_t *agparent(Agraph_t * g);
+/// @}
 
-/* set cardinality */
+/** @defgroup cgraph_misc miscellaneous
+ *  @ingroup cgraph_api
+ *  @{
+ */
+
+/// @defgroup card set cardinality
+/// @{
 CGRAPH_API int agnnodes(Agraph_t * g);
 CGRAPH_API int agnedges(Agraph_t * g);
 CGRAPH_API int agnsubg(Agraph_t * g);
 CGRAPH_API int agdegree(Agraph_t * g, Agnode_t * n, int in, int out);
 CGRAPH_API int agcountuniqedges(Agraph_t * g, Agnode_t * n, int in, int out);
+/// @}
 
-/* memory */
+/// @defgroup cgmem memory
+/// @{
 CGRAPH_API void *agalloc(Agraph_t * g, size_t size);
 CGRAPH_API void *agrealloc(Agraph_t * g, void *ptr, size_t oldsize,
 		       size_t size);
@@ -406,6 +661,7 @@ CGRAPH_API void aginternalmapclearlocalnames(Agraph_t * g);
 
 #define agnew(g,t)		((t*)agalloc(g,sizeof(t)))
 #define agnnew(g,n,t)	((t*)agalloc(g,(n)*sizeof(t)))
+/// @}
 
 /// @cond
 
@@ -418,7 +674,8 @@ CGRAPH_API void aginternalmapclearlocalnames(Agraph_t * g);
 
 /// @endcond
 
-/* error handling */
+/// @defgroup cgraph_err error handling
+/// @{
 typedef enum { AGWARN, AGERR, AGMAX, AGPREV } agerrlevel_t;
 typedef int (*agusererrf) (char*);
 CGRAPH_API agerrlevel_t agseterr(agerrlevel_t);
@@ -430,13 +687,14 @@ CGRAPH_API void agwarningf(const char *fmt, ...) PRINTF_LIKE(1, 2);
 CGRAPH_API int agerrors(void);
 CGRAPH_API int agreseterrors(void);
 CGRAPH_API agusererrf agseterrf(agusererrf);
+/// @}
 
 #undef PRINTF_LIKE
 
 /* data access macros */
-/* this assumes that e[0] is out and e[1] is inedge, see edgepair in edge.c  */
-#define AGIN2OUT(e)		((e)-1)
-#define AGOUT2IN(e)		((e)+1)
+/* this assumes that e[0] is out and e[1] is inedge, see @ref Agedgepair_s  */
+#define AGIN2OUT(inedge)		((inedge)-1) ///< Agedgepair_s.in -> Agedgepair_s.out
+#define AGOUT2IN(outedge)		((outedge)+1) ///< Agedgepair_s.out -> Agedgepair_s.in
 #define AGOPP(e)		((AGTYPE(e)==AGINEDGE)?AGIN2OUT(e):AGOUT2IN(e))
 #define AGMKOUT(e)		(AGTYPE(e) == AGOUTEDGE? (e): AGIN2OUT(e))
 #define AGMKIN(e)		(AGTYPE(e) == AGINEDGE?  (e): AGOUT2IN(e))
@@ -446,46 +704,99 @@ CGRAPH_API agusererrf agseterrf(agusererrf);
 /* These macros are also exposed as functions, so they can be linked against. */
 #define agtail(e)		AGTAIL(e)
 #define aghead(e)		AGHEAD(e)
-#define agopp(e)		AGOPP(e)
-#define ageqedge(e,f)		AGEQEDGE(e,f)
+#define agopp(e)		AGOPP(e) ///< opposite edge: flip Agedgepair_s.out ⇄ Agedgepair_s.in
+#define ageqedge(e,f)		AGEQEDGE(e,f) ///< edges are equal
 
 #define TAILPORT_ID		"tailport"
 #define HEADPORT_ID		"headport"
+/// @}
 
-CGRAPH_API extern Agdesc_t Agdirected;
+/// @addtogroup cgraph_graph
+/// @{
+CGRAPH_API extern Agdesc_t Agdirected; ///< directed
 CGRAPH_API extern Agdesc_t Agstrictdirected;
-CGRAPH_API extern Agdesc_t Agundirected;
-CGRAPH_API extern Agdesc_t Agstrictundirected;
+///< strict directed. A strict graph cannot have multi-edges or self-arcs.
+CGRAPH_API extern Agdesc_t Agundirected; ///< undirected
+CGRAPH_API extern Agdesc_t Agstrictundirected; ///< strict undirected
+/// @}
 
-/* fast graphs */
-void agflatten(Agraph_t * g, int flag);
-typedef Agsubnode_t	Agnoderef_t;
-typedef Dtlink_t	Agedgeref_t;
+/** @defgroup cgraph_fast fast graphs
+ *  @ingroup cgraph_misc
+ *  @{
+ */
 
-#define AGHEADPOINTER(g)	((Agnoderef_t*)(g->n_seq->data->hh._head))
-#define AGRIGHTPOINTER(rep)  	((Agnoderef_t*)((rep)->seq_link.right?((void*)((rep)->seq_link.right) - offsetof(Agsubnode_t,seq_link)):0))
-#define AGLEFTPOINTER(rep)  	((Agnoderef_t*)((rep)->seq_link.hl._left?((void*)((rep)->seq_link.hl._left) - offsetof(Agsubnode_t,seq_link)):0))
-
-#define FIRSTNREF(g)	(agflatten(g,1), AGHEADPOINTER(g))
-
-#define NEXTNREF(g,rep)  	(AGRIGHTPOINTER(rep) == AGHEADPOINTER(g)?0:AGRIGHTPOINTER(rep))
-
-#define PREVNREF(g,rep)  	(((rep)==AGHEADPOINTER(g))?0:(AGLEFTPOINTER(rep)))
-
-#define LASTNREF(g)		(agflatten(g,1), AGHEADPOINTER(g)?AGLEFTPOINTER(AGHEADPOINTER(g)):0)
-#define NODEOF(rep)		((rep)->node)
-
-#define FIRSTOUTREF(g,sn)	(agflatten(g,1), (sn)->out_seq)
-#define LASTOUTREF(g,sn)	(agflatten(g,1), (Agedgeref_t*)dtlast(sn->out_seq))
-#define FIRSTINREF(g,sn)	(agflatten(g,1), (sn)->in_seq)
-#define NEXTEREF(g,rep)  	((rep)->right)
-#define PREVEREF(g,rep)  	((rep)->hl._left)
 /* this is expedient but a bit slimey because it "knows" that dict entries of both nodes
 and edges are embedded in main graph objects but allocated separately in subgraphs */
 #define AGSNMAIN(sn)        ((sn)==(&((sn)->node->mainsub)))
 #define EDGEOF(sn,rep)		(AGSNMAIN(sn)?((Agedge_t*)((unsigned char*)(rep) - offsetof(Agedge_t,seq_link))) : ((Dthold_t*)(rep))->obj)
+/// @}
+
+/// options for passing to `graphviz_acyclic`
+typedef struct {
+  FILE *outFile;
+  bool doWrite;
+  bool Verbose;
+} graphviz_acyclic_options_t;
+
+/// programmatic access to `acyclic`
+///
+/// See `man acyclic` for an explanation of the `acyclic` tool.
+///
+/// \param g Graph to operate on
+/// \param opts Options to control acyclic algorithm
+/// \param num_rev [inout] Running total of reversed edges
+/// \return True if a cycle was found, indicating failure
+CGRAPH_API bool graphviz_acyclic(Agraph_t *g,
+                                 const graphviz_acyclic_options_t *opts,
+                                 size_t *num_rev);
+
+/// options for passing to `graphviz_tred`
+typedef struct {
+  bool Verbose;
+  bool PrintRemovedEdges;
+  FILE *out; ///< stream to write result(s) to
+  FILE *err; ///< stream to print warnings to
+} graphviz_tred_options_t;
+
+/// programmatic access to `tred`
+///
+/// See `man tred` for an explanation of the `tred` tool.
+///
+/// \param g Graph to operate on
+/// \param opts Options to control tred algorithm
+CGRAPH_API void graphviz_tred(Agraph_t *g, const graphviz_tred_options_t *opts);
+
+/// options for passing to `graphviz_unflatten`
+typedef struct {
+  bool Do_fans;
+  int MaxMinlen;
+  int ChainLimit;
+} graphviz_unflatten_options_t;
+
+/// programmatic access to `unflatten`
+///
+/// See `man unflatten` for an explanation of the `unflatten` tool.
+///
+/// \param g Graph to operate on
+/// \param opts Options to control unflattening
+CGRAPH_API void graphviz_unflatten(Agraph_t *g,
+                                   const graphviz_unflatten_options_t *opts);
+
+/** add to a graph any edges with both endpoints within that graph
+ *
+ * If `edgeset` is given as `NULL`, edges from the root graph of `g` will be
+ * considered. In this case if `g` itself is the root graph, this call is a
+ * no-op.
+ *
+ * If `g` is a connected component, the edges added will be all edges attached
+ * to any node in `g`.
+ *
+ * \param g Graph to add edges to
+ * \param edgeset Graph whose edges to consider
+ * \return Number of edges added
+ */
+CGRAPH_API size_t graphviz_node_induce(Agraph_t *g, Agraph_t *edgeset);
 
 #ifdef __cplusplus
 }
 #endif
-/// @}
