@@ -1,6 +1,5 @@
-/* $Id: gvcommon.h,v 1.15 2011/01/25 16:30:49 ellson Exp $ $Revision: 1.15 $ */
-/* vim:set shiftwidth=4 ts=8: */
-
+/// @file
+/// @ingroup gvc_api
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -8,11 +7,10 @@
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#ifndef GVCOMMON_H
-#define GVCOMMON_H
+#pragma once
 
 #ifdef __cplusplus
 extern "C" {
@@ -22,7 +20,7 @@ extern "C" {
 	char **info;
 	char *cmdname;
 	int verbose;
-	boolean config, auto_outfile_names;
+	bool config, auto_outfile_names;
         void (*errorfn) (const char *fmt, ...);
 	const char **show_boxes; /* emit code for correct box coordinates */
 	const char **lib; 
@@ -37,4 +35,3 @@ extern "C" {
 #ifdef __cplusplus
 }
 #endif
-#endif				/* GVCOMMON_H */
