@@ -1,6 +1,8 @@
-/* $Id: gvplugin.h,v 1.32 2011/01/25 16:30:49 ellson Exp $ $Revision: 1.32 $ */
-/* vim:set shiftwidth=4 ts=8: */
-
+/// @file
+/// @ingroup plugin_api
+/// @defgroup plugin_api Plugin API
+/// @ingroup public_apis
+/// @{
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -8,13 +10,12 @@
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 /* Header used by plugins */
 
-#ifndef GVPLUGIN_H
-#define GVPLUGIN_H
+#pragma once
 
 #ifdef __cplusplus
 extern "C" {
@@ -30,6 +31,7 @@ extern "C" {
  *          type      - e.g. "png", "ps"
  */
 
+/// ingroup plugin_api
     typedef struct {
 	int id;         /* an id that is only unique within a package 
 			of plugins of the same api.
@@ -37,7 +39,7 @@ extern "C" {
 			has an id that is different from the "ps" type
 			in the same package */
 	const char *type;	/* a string name, such as "png" or "ps" that
-			distinguishes different types withing the same
+			distinguishes different types within the same
 			 (renderer in this case) */
 	int quality;    /* an arbitrary integer used for ordering plugins of
 			the same type from different packages */
@@ -56,8 +58,8 @@ extern "C" {
 					no pathname */
 	gvplugin_api_t *apis;
     } gvplugin_library_t;
+/// @}
 
 #ifdef __cplusplus
 }
 #endif
-#endif				/* GVPLUGIN_H */
