@@ -1,6 +1,5 @@
-/* $Id: gvplugin_textlayout.h,v 1.17 2011/01/25 16:30:49 ellson Exp $ $Revision: 1.17 $ */
-/* vim:set shiftwidth=4 ts=8: */
-
+/// @file
+/// @ingroup plugin_api
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -8,26 +7,26 @@
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#ifndef GVPLUGIN_TEXTLAYOUT_H
-#define GVPLUGIN_TEXTLAYOUT_H
+#pragma once
 
 #include "types.h"
 #include "gvplugin.h"
 #include "gvcjob.h"
 #include "gvcommon.h"
+#include <stdbool.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+    /// @ingroup plugin_api
     struct gvtextlayout_engine_s {
-	boolean (*textlayout) (textpara_t *para, char** fontpath);
+	bool (*textlayout) (textspan_t *span, char** fontpath);
     };
 
 #ifdef __cplusplus
 }
 #endif
-#endif				/* GVPLUGIN_TEXTLAYOUT_H */
